@@ -3,9 +3,7 @@
 extern char *HYDRA_EXIT;
 char *buf;
 
-int
-start_cisco_enable(int s, unsigned long int ip, int port, unsigned char options, char *miscptr, FILE * fp)
-{
+int start_cisco_enable(int s, char *ip, int port, unsigned char options, char *miscptr, FILE * fp) {
   char *empty = "";
   char *pass, buffer[300];
 
@@ -17,7 +15,7 @@ start_cisco_enable(int s, unsigned long int ip, int port, unsigned char options,
     return 1;
   }
   buf = hydra_receive_line(s);
-  if (strstr(buf, "assw") != NULL) {
+  if (buf != NULL && strstr(buf, "assw") != NULL) {
     hydra_completed_pair();
     free(buf);
     if (memcmp(hydra_get_next_pair(), &HYDRA_EXIT, sizeof(HYDRA_EXIT)) == 0)
@@ -44,7 +42,8 @@ start_cisco_enable(int s, unsigned long int ip, int port, unsigned char options,
     }
   }
 
-  if (strstr(buf, "assw") != NULL || strstr(buf, "ad ") != NULL || strstr(buf, "attempt") != NULL || strstr(buf, "fail") != NULL || strstr(buf, "denied") != NULL) {
+  if (buf != NULL
+      && (strstr(buf, "assw") != NULL || strstr(buf, "ad ") != NULL || strstr(buf, "attempt") != NULL || strstr(buf, "fail") != NULL || strstr(buf, "denied") != NULL)) {
     free(buf);
     hydra_completed_pair();
     if (memcmp(hydra_get_next_pair(), &HYDRA_EXIT, sizeof(HYDRA_EXIT)) == 0)
@@ -52,15 +51,15 @@ start_cisco_enable(int s, unsigned long int ip, int port, unsigned char options,
     return 2;
   }
 
+  if (buf != NULL)
+    free(buf);
   hydra_report_found_host(port, ip, "cisco-enable", fp);
   hydra_completed_pair_found();
   return 3;
 }
 
-void
-service_cisco_enable(unsigned long int ip, int sp, unsigned char options, char *miscptr, FILE * fp, int port)
-{
-  int run = 1, failc = 0, retry = 1, next_run, sock = -1;
+void service_cisco_enable(char *ip, int sp, unsigned char options, char *miscptr, FILE * fp, int port) {
+  int run = 1, failc = 0, retry = 1, next_run = 1, sock = -1;
   int myport = PORT_TELNET, mysslport = PORT_TELNET_SSL;
   char buffer[300];
   char *login;
@@ -75,7 +74,7 @@ service_cisco_enable(unsigned long int ip, int sp, unsigned char options, char *
       {
         if (sock >= 0)
           sock = hydra_disconnect(sock);
-        usleep(275000);
+//        usleep(275000);
         if ((options & OPTION_SSL) == 0) {
           if (port != 0)
             myport = port;
@@ -88,36 +87,48 @@ service_cisco_enable(unsigned long int ip, int sp, unsigned char options, char *
           port = mysslport;
         }
         if (sock < 0) {
-          fprintf(stderr, "Error: Child with pid %d terminating, can not connect\n", (int) getpid());
+          fprintf(stderr, "[ERROR] Child with pid %d terminating, can not connect\n", (int) getpid());
           hydra_child_exit(1);
         }
 
         /* Cisco AAA Support */
         if (strlen(login = hydra_get_next_login()) != 0) {
-          while ((buf = hydra_receive_line(sock)) != NULL && strstr(buf, "name") == NULL) {
+          while ((buf = hydra_receive_line(sock)) != NULL && strstr(buf, "name:") == NULL && strstr(buf, "ogin:") == NULL) {
+            if (hydra_strcasestr(buf, "ress ENTER") != NULL)
+              hydra_send(sock, "\r\n", 2, 0);
             free(buf);
           }
 
-          sprintf(buffer, "%.250s\r", login);
-          if (hydra_send(sock, buffer, strlen(buffer) + 1, 0) < 0) {
-            fprintf(stderr, "Error: Child with pid %d terminating, can not send login\n", (int) getpid());
+          sprintf(buffer, "%.250s\r\n", login);
+          if (hydra_send(sock, buffer, strlen(buffer), 0) < 0) {
+            fprintf(stderr, "[ERROR] Child with pid %d terminating, can not send login\n", (int) getpid());
             hydra_child_exit(2);
           }
         }
 
-        while ((buf = hydra_receive_line(sock)) != NULL && strstr(buf, "assw") == NULL) {
-          free(buf);
-        }
+        if (miscptr != NULL) {
+          while ((buf = hydra_receive_line(sock)) != NULL && strstr(buf, "assw") == NULL) {
+            if (hydra_strcasestr(buf, "ress ENTER") != NULL)
+              hydra_send(sock, "\r\n", 2, 0);
+            free(buf);
+          }
 
-        sprintf(buffer, "%.250s\r\n", miscptr);
-        if (hydra_send(sock, buffer, strlen(buffer), 0) < 0) {
-          fprintf(stderr, "Error: Child with pid %d terminating, can not send login\n", (int) getpid());
-          hydra_child_exit(2);
+          sprintf(buffer, "%.250s\r\n", miscptr);
+          if (hydra_send(sock, buffer, strlen(buffer), 0) < 0) {
+            fprintf(stderr, "[ERROR] Child with pid %d terminating, can not send login\n", (int) getpid());
+            hydra_child_exit(2);
+          }
         }
 
         buf = hydra_receive_line(sock);
+        if (hydra_strcasestr(buf, "ress ENTER") != NULL) {
+          hydra_send(sock, "\r\n", 2, 0);
+          free(buf);
+          buf = hydra_receive_line(sock);
+        }
+
         if (strstr(buf, "assw") != NULL) {
-          fprintf(stderr, "Error: Child with pid %d terminating - can not login, can not login\n", (int) getpid());
+          fprintf(stderr, "[ERROR] Child with pid %d terminating - can not login, can not login\n", (int) getpid());
           hydra_child_exit(2);
         }
         free(buf);
@@ -127,12 +138,12 @@ service_cisco_enable(unsigned long int ip, int sp, unsigned char options, char *
       }
     case 2:                    /* run the cracking function */
       {
-        unsigned char *buf2 = malloc(256);
+        unsigned char *buf2;
         int f = 0;
 
         sprintf(buffer, "%.250s\r\n", "ena");
         if (hydra_send(sock, buffer, strlen(buffer), 0) < 0) {
-          fprintf(stderr, "Error: Child with pid %d terminating, can not send 'ena'\n", (int) getpid());
+          fprintf(stderr, "[ERROR] Child with pid %d terminating, can not send 'ena'\n", (int) getpid());
           hydra_child_exit(2);
         }
 
@@ -145,11 +156,11 @@ service_cisco_enable(unsigned long int ip, int sp, unsigned char options, char *
             if (failc < retry) {
               next_run = 1;
               failc++;
-              fprintf(stderr, "Error: Child with pid %d was disconnected - retrying (%d of %d retries)\n", getpid(), failc, retry);
+              fprintf(stderr, "[ERROR] Child with pid %d was disconnected - retrying (%d of %d retries)\n", (int) getpid(), failc, retry);
               sleep(3);
               break;
             } else {
-              fprintf(stderr, "Error: Child with pid %d was disconnected - exiting\n", getpid());
+              fprintf(stderr, "[ERROR] Child with pid %d was disconnected - exiting\n", (int) getpid());
               hydra_child_exit(0);
             }
           }
@@ -165,7 +176,7 @@ service_cisco_enable(unsigned long int ip, int sp, unsigned char options, char *
     case 3:                    /* clean exit */
       sprintf(buffer, "%.250s\r\n", "exit");
       if (hydra_send(sock, buffer, strlen(buffer), 0) < 0) {
-        fprintf(stderr, "Error: Child with pid %d terminating, can not send 'exit'\n", (int) getpid());
+        fprintf(stderr, "[ERROR] Child with pid %d terminating, can not send 'exit'\n", (int) getpid());
         hydra_child_exit(0);
       }
       if (sock >= 0)
@@ -173,9 +184,9 @@ service_cisco_enable(unsigned long int ip, int sp, unsigned char options, char *
       hydra_child_exit(0);
       return;
     default:
-      fprintf(stderr, "Caught unknown return code, exiting!\n");
+      fprintf(stderr, "[ERROR] Caught unknown return code, exiting!\n");
       hydra_child_exit(0);
-      exit(-1);
+      hydra_child_exit(2);
     }
     run = next_run;
   }
