@@ -1,13 +1,13 @@
 #include "hydra-mod.h"
 
+// no memleaks found on 110425
+
 #define COMMAND "/bin/ls /"
 
 extern char *HYDRA_EXIT;
 char *buf;
 
-int
-start_rexec(int s, unsigned long int ip, int port, unsigned char options, char *miscptr, FILE * fp)
-{
+int start_rexec(int s, char *ip, int port, unsigned char options, char *miscptr, FILE * fp) {
   char *empty = "";
   char *login, *pass, buffer[300] = "", buffer2[100], *bptr = buffer2;
   int ret;
@@ -45,10 +45,8 @@ start_rexec(int s, unsigned long int ip, int port, unsigned char options, char *
   return 1;
 }
 
-void
-service_rexec(unsigned long int ip, int sp, unsigned char options, char *miscptr, FILE * fp, int port)
-{
-  int run = 1, next_run, sock = -1;
+void service_rexec(char *ip, int sp, unsigned char options, char *miscptr, FILE * fp, int port) {
+  int run = 1, next_run = 1, sock = -1;
   int myport = PORT_REXEC, mysslport = PORT_REXEC_SSL;
 
   hydra_register_socket(sp);
@@ -61,7 +59,7 @@ service_rexec(unsigned long int ip, int sp, unsigned char options, char *miscptr
       {
         if (sock >= 0)
           sock = hydra_disconnect(sock);
-        usleep(275000);
+//        usleep(275000);
         if ((options & OPTION_SSL) == 0) {
           if (port != 0)
             myport = port;
@@ -74,7 +72,7 @@ service_rexec(unsigned long int ip, int sp, unsigned char options, char *miscptr
           port = mysslport;
         }
         if (sock < 0) {
-          fprintf(stderr, "Error: Child with pid %d terminating, can not connect\n", (int) getpid());
+          hydra_report(stderr, "[ERROR] Child with pid %d terminating, can not connect\n", (int) getpid());
           hydra_child_exit(1);
         }
         next_run = 2;
@@ -89,9 +87,9 @@ service_rexec(unsigned long int ip, int sp, unsigned char options, char *miscptr
       hydra_child_exit(0);
       return;
     default:
-      fprintf(stderr, "Caught unknown return code, exiting!\n");
+      hydra_report(stderr, "[ERROR] Caught unknown return code, exiting!\n");
       hydra_child_exit(0);
-      exit(-1);
+
     }
     run = next_run;
   }
